@@ -2,28 +2,35 @@
 # encoding: utf-8
 
 __author__ = "ChenyangGao <https://chenyanggao.github.io>"
-__version__ = (0, 1, 2)
+__version__ = (0, 1, 4)
 __all__ = [
     "Return", "Yield", "YieldFrom", "iterable", "async_iterable", "foreach", "async_foreach", 
-    "through", "async_through", "flatten", "async_flatten", "iter_unique", "async_iter_unique", 
-    "chunked", "wrap_iter", "wrap_aiter", "acc_step", "cut_iter", "run_gen_step", 
-    "run_gen_step_iter", "bfs_gen", 
+    "through", "async_through", "flatten", "async_flatten", "map", "filter", "reduce", "zip", 
+    "chunked", "iter_unique", "async_iter_unique", "wrap_iter", "wrap_aiter", "acc_step", 
+    "cut_iter", "run_gen_step", "run_gen_step_iter", "bfs_gen", "with_iter_next", "backgroud_loop", 
 ]
 
-
 from abc import ABC, abstractmethod
-from asyncio import to_thread
+from asyncio import create_task, sleep as async_sleep, to_thread
+from builtins import map as _map, filter as _filter, zip as _zip
 from collections import deque
 from collections.abc import (
-    AsyncIterable, AsyncIterator, Buffer, Callable, Generator, 
+    AsyncIterable, AsyncIterator, Awaitable, Buffer, Callable, Generator, 
     Iterable, Iterator, MutableSet, Sequence, 
 )
+from contextlib import asynccontextmanager, contextmanager
 from dataclasses import dataclass
 from itertools import batched, pairwise
-from inspect import isawaitable
-from typing import overload, Any, Literal
+from inspect import isawaitable, iscoroutinefunction
+from _thread import start_new_thread
+from time import sleep, time
+from typing import overload, Any, AsyncContextManager, ContextManager, Literal
 
-from asynctools import async_map, async_zip, async_batched, ensure_async, ensure_aiter
+from asynctools import (
+    async_filter, async_map, async_reduce, async_zip, async_batched, ensure_async, ensure_aiter, 
+)
+from texttools import format_time
+from undefined import undefined, Undefined
 
 
 @dataclass(slots=True, frozen=True)
@@ -73,7 +80,7 @@ def foreach(
     if not (isinstance(iterable, Iterable) and all(isinstance(it, Iterable) for it in iterables)):
         return async_foreach(value, iterable, *iterables)
     if iterables:
-        for args in zip(iterable, *iterables):
+        for args in _zip(iterable, *iterables):
             value(*args)
     else:
         for arg in iterable:
@@ -107,7 +114,7 @@ def through(
         for _ in iterable:
             pass
     else:
-        for v in map(take_while, iterable):
+        for v in _map(take_while, iterable):
             if not v:
                 break
 
@@ -176,6 +183,96 @@ async def async_flatten(
             yield e
 
 
+def map(
+    function: None | Callable, 
+    iterable: Iterable | AsyncIterable, 
+    /, 
+    *iterables: Iterable | AsyncIterable, 
+):      
+    if (
+        iscoroutinefunction(function) or 
+        isinstance(iterable, AsyncIterable) or 
+        any(isinstance(i, AsyncIterable) for i in iterables)
+    ):
+        if function is None:
+            if iterables:
+                return async_zip(iterable, *iterables)
+            else:
+                return iterable
+        return async_map(function, iterable, *iterables)
+    if function is None:
+        if iterables:
+            return _zip(iterable, *iterables)
+        else:
+            return iterable
+    return _map(function, iterable, *iterables)
+
+
+def filter(
+    function: None | Callable, 
+    iterable: Iterable | AsyncIterable, 
+    /, 
+):
+    if iscoroutinefunction(function) or isinstance(iterable, AsyncIterable):
+        return async_filter(function, iterable)
+    return _filter(function, iterable)
+
+
+def reduce(
+    function: Callable, 
+    iterable: Iterable | AsyncIterable, 
+    initial = undefined, 
+    /, 
+):
+    if iscoroutinefunction(function) or isinstance(iterable, AsyncIterable):
+        return async_reduce(function, iterable, initial)
+    from functools import reduce
+    if initial is undefined:
+        return reduce(function, iterable)
+    return reduce(function, iterable, initial)
+
+
+def zip(
+    iterable: Iterable | AsyncIterable, 
+    /, 
+    *iterables: Iterable | AsyncIterable, 
+):
+    if isinstance(iterable, AsyncIterable) or any(isinstance(i, AsyncIterable) for i in iterables):
+        return async_zip(iterable, *iterables)
+    return _zip(iterable, *iterables)
+
+
+@overload
+def chunked[T](
+    iterable: Iterable[T], 
+    n: int = 1, 
+    /, 
+) -> Iterator[Sequence[T]]:
+    ...
+@overload
+def chunked[T](
+    iterable: AsyncIterable[T], 
+    n: int = 1, 
+    /, 
+) -> AsyncIterator[Sequence[T]]:
+    ...
+def chunked[T](
+    iterable: Iterable[T] | AsyncIterable[T], 
+    n: int = 1, 
+    /, 
+) -> Iterator[Sequence[T]] | AsyncIterator[Sequence[T]]:
+    if n < 0:
+        n = 1
+    if isinstance(iterable, Sequence):
+        if n == 1:
+            return ((e,) for e in iterable)
+        return (iterable[i:j] for i, j in pairwise(range(0, len(iterable)+n, n)))
+    elif isinstance(iterable, Iterable):
+        return batched(iterable, n)
+    else:
+        return async_batched(iterable, n)
+
+
 @overload
 def iter_unique[T](
     iterable: Iterable[T], 
@@ -223,37 +320,6 @@ async def async_iter_unique[T](
             add(e)
 
 
-@overload
-def chunked[T](
-    iterable: Iterable[T], 
-    n: int = 1, 
-    /, 
-) -> Iterator[Sequence[T]]:
-    ...
-@overload
-def chunked[T](
-    iterable: AsyncIterable[T], 
-    n: int = 1, 
-    /, 
-) -> AsyncIterator[Sequence[T]]:
-    ...
-def chunked[T](
-    iterable: Iterable[T] | AsyncIterable[T], 
-    n: int = 1, 
-    /, 
-) -> Iterator[Sequence[T]] | AsyncIterator[Sequence[T]]:
-    if n < 0:
-        n = 1
-    if isinstance(iterable, Sequence):
-        if n == 1:
-            return ((e,) for e in iterable)
-        return (iterable[i:j] for i, j in pairwise(range(0, len(iterable)+n, n)))
-    elif isinstance(iterable, Iterable):
-        return batched(iterable, n)
-    else:
-        return async_batched(iterable, n)
-
-
 @overload
 def wrap_iter[T](
     iterable: Iterable[T], 
@@ -656,3 +722,115 @@ def bfs_gen[T](
                 push(args)
                 args = yield val
 
+
+@overload
+def with_iter_next[T](
+    iterable: Iterable[T], 
+    /, 
+    async_: Literal[False] = False, 
+) -> ContextManager[Callable[[], T]]:
+    ...
+@overload
+def with_iter_next[T](
+    iterable: AsyncIterable[T], 
+    /, 
+    async_: Literal[False] = False, 
+) -> ContextManager[Callable[[], Awaitable[T]]]:
+    ...
+@overload
+def with_iter_next[T](
+    iterable: Iterable[T] | AsyncIterable[T], 
+    /, 
+    async_: Literal[True], 
+) -> ContextManager[Callable[[], Awaitable[T]]]:
+    ...
+@contextmanager
+def with_iter_next[T](
+    iterable: Iterable[T] | AsyncIterable[T], 
+    /, 
+    async_: Literal[False, True] = False, 
+):
+    if async_:
+        get_next: Callable[[], T] | Callable[[], Awaitable[T]] = ensure_aiter(iterable).__anext__
+    elif isinstance(iterable, Iterable):
+        get_next = iter(iterable).__next__
+    else:
+        get_next = aiter(iterable).__anext__
+        async_ = True
+    if async_:
+        try:
+            yield get_next
+        except StopAsyncIteration:
+            pass
+    else:
+        try:
+            yield get_next
+        except StopIteration:
+            pass
+
+
+@overload
+def backgroud_loop(
+    call: None | Callable = None, 
+    /, 
+    interval: int | float = 0.05, 
+    *, 
+    async_: Literal[False] = False, 
+) -> ContextManager:
+    ...
+@overload
+def backgroud_loop(
+    call: None | Callable = None, 
+    /, 
+    interval: int | float = 0.05, 
+    *, 
+    async_: Literal[True], 
+) -> AsyncContextManager:
+    ...
+def backgroud_loop(
+    call: None | Callable = None, 
+    /, 
+    interval: int | float = 0.05, 
+    *, 
+    async_: Literal[False, True] = False, 
+) -> ContextManager | AsyncContextManager:
+    use_default_call = not callable(call)
+    if use_default_call:
+        start = time()
+        def call():
+            print(f"\r\x1b[K{format_time(time() - start)}", end="")
+    def run():
+        while running:
+            try:
+                yield call
+            except Exception:
+                pass
+            if interval > 0:
+                if async_:
+                    yield async_sleep(interval)
+                else:
+                    sleep(interval)
+    running = True
+    if async_:
+        async def actx():
+            nonlocal running
+            try:
+                task = create_task(run())
+                yield task
+            finally:
+                running = False
+                task.cancel()
+                if use_default_call:
+                    print("\r\x1b[K", end="")
+        return asynccontextmanager(actx)()
+    else:
+        def ctx():
+            nonlocal running
+            try:
+                yield start_new_thread(run, ())
+            finally:
+                running = False
+                if use_default_call:
+                    print("\r\x1b[K", end="")
+        return contextmanager(ctx)()
+
